REM Oracle Stats compare
REM 
REM Installation script
REM
REM Requirements:
REM	- Run as sysdba

/*

	This script can be run even if you dont have an account with Stats Compare.
	The following example shows how to:
	SQL> exec util_run_compare.start_compare;
	SQL> <your first query here>
	SQL> exec util_run_compare.middle_compare;
	SQL> <your second query here>
	SQL> exec util_run_compare.end_compare;
	SQL> set lines 150
	SQL> select util_run_compare.get_access_url as "Paste to browser address bar" from dual;

*/

set define %

accept use_proxy default 'none' prompt 'HTTP Proxy server [none]: '
accept use_proxy_port default 8080 prompt 'HTTP Proxy port [8080]: '
accept use_name default 'none' prompt 'User you would like to use Stats Compare: '

/* set echo off;
set feedback off;
set show off;
set term off; */

declare
	acl_count	number;
begin
	$if dbms_db_version.ver_le_10 = false $then

		select count(acl)
		into acl_count
		from dba_network_acls
		where acl = '/sys/acls/db_ping.xml';

		if acl_count = 0 then
			dbms_network_acl_admin.create_acl (
				acl          => 'db_ping.xml',
				description  => 'Permissions to access db ping servers',
				principal    => upper('%%use_name'),
				is_grant     => true,
				privilege    => 'connect'
			);
			commit;
		else
			dbms_network_acl_admin.add_privilege (
				acl	 => 'db_ping.xml',
				principal	 => upper('%%use_name'),
				is_grant	 => true,
				privilege	 => 'connect'
			);
			commit;
		end if;

		dbms_network_acl_admin.add_privilege (
			acl	 => 'db_ping.xml',
			principal	 => upper('%%use_name'),
			is_grant	 => true,
			privilege	 => 'resolve'
		);
		commit;

		dbms_network_acl_admin.assign_acl (
			acl          => 'db_ping.xml',
			host         => 'www.dbping.com',
			lower_port	 => 80,
			upper_port	 => 80
		);
		commit;

		if '%%use_proxy' != 'none' then
			dbms_network_acl_admin.assign_acl (
				acl          => 'db_ping.xml',
				host         => '%%use_proxy',
				lower_port	 => %%use_proxy_port,
				upper_port	 => %%use_proxy_port
			);
			commit;
		end if;
	$else
		null;
	$end

	exception
		when others then
			null;
end;
/

grant select on sys.v_$statname to %%use_name;
grant select on sys.v_$mystat to %%use_name;
grant select on sys.v_$sesstat to %%use_name;
grant select on sys.v_$latch to %%use_name;
grant select on sys.v_$sess_time_model to %%use_name;

create or replace package %%use_name..util_run_compare

as

	procedure start_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	);

	procedure middle_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	);

	procedure end_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	);

	function get_access_url
	return varchar2;

end util_run_compare;
/

show errors;

create or replace package body %%use_name..util_run_compare

as

	g_debug_flag			boolean := false;

	g_transfer_url_f		varchar2(4000) := 'http://www.dbping.com/stats/!util_compare.transfer_f';
	g_draw_token_url		varchar2(4000) := 'http://www.dbping.com/stats/util_compare.draw_token';
	g_token_taken			varchar2(200);
	g_is_fast_draw			boolean := false;

	procedure do_debug(
		d_string		varchar2
	)

	as

	begin

		if g_debug_flag then
			dbms_output.put_line(d_string);
		end if;

	end do_debug;

	procedure transfer_f_compare_data(
		transfer_key		varchar2,
		transfer_settings	varchar2,
		transfer_from_sid	number default 0
	)

	as

		l_request		utl_http.req;
		l_response		utl_http.resp;
		l_response_val		varchar2(1024);
		l_param			clob;
		l_param_length		number;
		l_stat_string		varchar2(4000);

		l_offset		integer := 0;
		l_buffer		varchar2(30000);
		l_amount		number := 30000;

		cursor get_stat_vals(what_sid_in number) is
			select
				n.class o_class_id,
				n.name o_name,
				s.statistic# o_identifier,
				s.value o_value
			from 
				v$statname n
				, v$sesstat s
			where 
				n.statistic# = s.statistic#
			and
				s.sid = what_sid_in
		union all
			select
				42,
				name,
				latch#,
				gets
			from
				v$latch
		union all
			select
				10042
				, stat_name
				, stat_id
				, value
			from
				v$sess_time_model
			where
				sid = what_sid_in;

		set_sid			number;

	begin

		if transfer_from_sid = 0 then
			select distinct sid
			into set_sid
			from v$mystat;
		else
			set_sid := transfer_from_sid;
		end if;

		dbms_lob.createtemporary(l_param, true);

		-- first set key and settings parameters
		l_stat_string := 'key='|| utl_url.escape(transfer_key) ||'&settings='|| utl_url.escape(transfer_settings);
		do_debug(l_stat_string);
		dbms_lob.writeappend(l_param, length(l_stat_string), l_stat_string);

		-- Set all others
		for stats in get_stat_vals(set_sid) loop
			l_stat_string := '&o_s_'|| stats.o_identifier ||'='|| stats.o_identifier ||'|'|| stats.o_name ||'|'|| stats.o_value ||'|'|| stats.o_class_id;
			dbms_lob.writeappend(l_param, length(l_stat_string), l_stat_string);
		end loop;

		l_param_length := length(l_param);

		l_request := utl_http.begin_request(
			url	=>	g_transfer_url_f,
			method	=>	'POST'
		);
		utl_http.set_header(
			r	=>	l_request,
			name	=>	'Content-Type',
			value	=>	'application/x-www-form-urlencoded'
		);
		utl_http.set_header(
			r	=>	l_request,
			name	=>	'Content-Length',
			value	=>	l_param_length
		);

		while (l_offset < l_param_length) loop
			if ( (l_param_length - l_offset) < l_amount ) then
				utl_http.write_text(
					r      =>   l_request,
					data   =>   substr(l_param, l_offset, l_param_length - l_offset)
				);
			else
				utl_http.write_text(
					r      =>   l_request,
					data   =>   substr(l_param, l_offset, l_amount)
				);
			end if;
			l_offset := l_offset + l_amount;
		end loop;

		l_response := utl_http.get_response(l_request);

		loop
			utl_http.read_line(l_response, l_response_val, true);
			-- dbms_output.put_line(l_response_val);
		end loop;
		
		utl_http.end_response(l_response);

		dbms_lob.freetemporary(l_param);

		if l_response_val != 0 then
			raise_application_error(-20042, 'Stats server failed to service your request');
		end if;
		
		exception
			when utl_http.end_of_body then
				utl_http.end_response(l_response);
				dbms_lob.freetemporary(l_param);


	end transfer_f_compare_data;

	procedure draw_token

	as

		token_drawn		varchar2(4000);

	begin

		select utl_http.request(g_draw_token_url)
		into token_drawn
		from dual;

		g_token_taken := replace(replace(token_drawn,chr(10)),chr(13));
		g_is_fast_draw := true;

		do_debug('Key received: ' || g_token_taken);

	end draw_token;

	procedure start_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	)

	as

		l_transfer_settings	varchar2(4000) := 'type:start;run_time:'|| to_char(sysdate,'DD-MM-YYYY-HH24-MI-SS');

	begin

		if options_in is not null then
			l_transfer_settings := l_transfer_settings || ';' || options_in;
		end if;

		if key_in is null and g_is_fast_draw = false then
			draw_token;
		end if;

		if key_in is null and g_is_fast_draw then
			do_debug('Calling with token: ' || g_token_taken);
			transfer_f_compare_data(g_token_taken, l_transfer_settings, for_session);
		else
			transfer_f_compare_data(key_in, l_transfer_settings, for_session);
		end if;

	end start_compare;

	procedure middle_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	)

	as

		l_transfer_settings	varchar2(4000) := 'type:middle;run_time:'|| to_char(sysdate,'DD-MM-YYYY-HH24-MI-SS');

	begin

		if options_in is not null then
			l_transfer_settings := l_transfer_settings || ';' || options_in;
		end if;

		if key_in is null and g_is_fast_draw then
			do_debug('Calling with token: ' || g_token_taken);
			transfer_f_compare_data(g_token_taken, l_transfer_settings, for_session);
		else
			transfer_f_compare_data(key_in, l_transfer_settings, for_session);
		end if;

	end middle_compare;

	procedure end_compare(
		key_in			varchar2 default null,
		options_in		varchar2 default null,
		for_session		number default 0
	)

	as

		l_transfer_settings	varchar2(4000) := 'type:end;run_time:'|| to_char(sysdate,'DD-MM-YYYY-HH24-MI-SS');

	begin

		if options_in is not null then
			l_transfer_settings := l_transfer_settings || ';' || options_in;
		end if;

		if key_in is null and g_is_fast_draw then
			do_debug('Calling with token: ' || g_token_taken);
			transfer_f_compare_data(g_token_taken, l_transfer_settings, for_session);
			dbms_output.put_line('See result at http://www.dbping.com/stats/compare_ui.direct_result?fdt='|| g_token_taken);
		else
			transfer_f_compare_data(key_in, l_transfer_settings, for_session);
		end if;

	end end_compare;

	function get_access_url
	return varchar2

	as

	begin

		if g_is_fast_draw then
			return 'See result at http://www.dbping.com/stats/compare_ui.direct_result?fdt='|| g_token_taken;
		else
			return 'No token drawn, please run start_compare at least once';
		end if;

	end get_access_url;

end util_run_compare;
/

show errors;