-- Show off dbms_parallel_execution

begin
	util_run_compare.start_compare('morten@egan.dk', 'title:REDGATE dbms_parallel_execute;name:Without parallel');
end;
/

update big_table set PHEROMONE_TREND_GUESS = PHEROMONE_TREND_GUESS * 0.2;

begin
	util_run_compare.middle_compare('morten@egan.dk', 'name:With parallel 4');
end;
/


-- ROWID chunks
DECLARE
  l_sql_stmt VARCHAR2(1000);
  l_try NUMBER;
  l_status NUMBER;
BEGIN
 
  -- Create the TASK
  DBMS_PARALLEL_EXECUTE.CREATE_TASK ('TASK_1_SIMPLE');
 
  -- Chunk the table by ROWID
  DBMS_PARALLEL_EXECUTE.CREATE_CHUNKS_BY_ROWID('TASK_1_SIMPLE', 'REDGATE', 'BIG_TABLE', true, 500);
 
  -- Execute the DML in parallel
  l_sql_stmt := 'update /*+ ROWID (dda) */ BIG_TABLE e 
      SET e.PHEROMONE_TREND_GUESS = e.PHEROMONE_TREND_GUESS * 0.2
      WHERE rowid BETWEEN :start_id AND :end_id';
  DBMS_PARALLEL_EXECUTE.RUN_TASK('TASK_1_SIMPLE', l_sql_stmt, DBMS_SQL.NATIVE,
                                 parallel_level => 4);
 
  -- If there is an error, RESUME it for at most 2 times.
  L_try := 0;
  L_status := DBMS_PARALLEL_EXECUTE.TASK_STATUS('TASK_1_SIMPLE');
  WHILE(l_try < 2 and L_status != DBMS_PARALLEL_EXECUTE.FINISHED) 
  LOOP
    L_try := l_try + 1;
    DBMS_PARALLEL_EXECUTE.RESUME_TASK('TASK_1_SIMPLE');
    L_status := DBMS_PARALLEL_EXECUTE.TASK_STATUS('TASK_1_SIMPLE');
  END LOOP;
 
  -- Done with processing; drop the task
  DBMS_PARALLEL_EXECUTE.DROP_TASK('TASK_1_SIMPLE');
   
END;
/

begin
	util_run_compare.end_compare('morten@egan.dk', 'name:With parallel 4');
end;
/

-- NUMBER chunks
DECLARE
  l_chunk_sql VARCHAR2(1000);
  l_sql_stmt VARCHAR2(1000);
  l_try NUMBER;
  l_status NUMBER;
BEGIN
 
  -- Create the TASK
  DBMS_PARALLEL_EXECUTE.CREATE_TASK ('TASK_2_SIMPLE');
 
  DBMS_PARALLEL_EXECUTE.CREATE_CHUNKS_BY_NUMBER_COL('TASK_2_SIMPLE', 'REDGATE', 'BIG_TABLE', 'PHEROMONE_TREND_GUESS', 10000);
 
  -- Execute the DML in parallel
  l_sql_stmt := 'update /*+ ROWID (dda) */ BIG_TABLE e 
      SET e.PHEROMONE_TREND_GUESS = e.PHEROMONE_TREND_GUESS * 0.2
      WHERE e.PHEROMONE_TREND_GUESS between :start_id and :end_id';
  DBMS_PARALLEL_EXECUTE.RUN_TASK('TASK_2_SIMPLE', l_sql_stmt, DBMS_SQL.NATIVE,
                                  parallel_level => 4);
 
  -- If there is error, RESUME it for at most 2 times.
  L_try := 0;
  L_status := DBMS_PARALLEL_EXECUTE.TASK_STATUS('TASK_2_SIMPLE');
  WHILE(l_try < 2 and L_status != DBMS_PARALLEL_EXECUTE.FINISHED) 
  Loop
    L_try := l_try + 1;
    DBMS_PARALLEL_EXECUTE.RESUME_TASK('TASK_2_SIMPLE');
    L_status := DBMS_PARALLEL_EXECUTE.TASK_STATUS('TASK_2_SIMPLE');
  END LOOP;
 
  -- Done with processing; drop the task
  DBMS_PARALLEL_EXECUTE.DROP_TASK('TASK_2_SIMPLE');
 
end;
/

-- MANUAL chunk execution
DECLARE
  l_sql_stmt varchar2(1000);
  l_try number;
  l_status number;
  l_chunk_id number;
  l_start_rowid rowid;
  l_end_rowid rowid;
  l_any_rows boolean;
  CURSOR c1 IS SELECT chunk_id
               FROM user_parallel_execute_chunks
               WHERE task_name = 'TASK_3_SIMPLE'
                 AND STATUS IN (DBMS_PARALLEL_EXECUTE.PROCESSED_WITH_ERROR,
                                DBMS_PARALLEL_EXECUTE.ASSIGNED);
BEGIN
 
  -- Create the Objects, task, and chunk by ROWID
  DBMS_PARALLEL_EXECUTE.CREATE_TASK ('TASK_3_SIMPLE');
  DBMS_PARALLEL_EXECUTE.CREATE_CHUNKS_BY_ROWID('TASK_3_SIMPLE', 'REDGATE', 'BIG_TABLE', true, 500);
 
  l_sql_stmt := 'update /*+ ROWID (dda) */ BIG_TABLE e 
      SET e.PHEROMONE_TREND_GUESS = e.PHEROMONE_TREND_GUESS * 0.2
      WHERE rowid BETWEEN :start_id AND :end_id';
 
  LOOP
      --
      -- Get a chunk to process; if there is nothing to process, then exit the 
      -- loop;
      --
      DBMS_PARALLEL_EXECUTE.GET_ROWID_CHUNK('TASK_3_SIMPLE',
                                                          l_chunk_id,
                                                          l_start_rowid,
                                                          l_end_rowid,
                                                          l_any_rows);
      IF (l_any_rows = false) THEN EXIT; END IF;

      BEGIN
        EXECUTE IMMEDIATE l_sql_stmt using l_start_rowid, l_end_rowid;
        DBMS_PARALLEL_EXECUTE.SET_CHUNK_STATUS('TASK_3_SIMPLE',l_chunk_id,
          DBMS_PARALLEL_EXECUTE.PROCESSED);
      EXCEPTION WHEN OTHERS THEN
        DBMS_PARALLEL_EXECUTE.SET_CHUNK_STATUS('TASK_3_SIMPLE', l_chunk_id,
          DBMS_PARALLEL_EXECUTE.PROCESSED_WITH_ERROR, SQLCODE, SQLERRM);
      END;
 
      --
      -- Finished processing one chunk; Commit here
      --
      COMMIT;
  END LOOP;

END;
/