-- Show utl_compress

create or replace function do_compress_content (
	text_in varchar2
)
return blob

as

	ret_blob	blob := empty_blob;
	l_blob		blob;

begin
	
	l_blob := utl_raw.cast_to_raw(text_in);
	dbms_lob.createtemporary(ret_blob, true);
	ret_blob := utl_compress.lz_compress(l_blob);

	return ret_blob;

end do_compress_content;
/

begin
	util_run_compare.start_compare('morten@egan.dk', 'title:REDGATE Compression insert;name:Insert without compression');
end;
/

create table no_compress as
	select nyhed_id, nyhed_titel, cast(substr(nyhed_content, 1, 4000) as varchar2(4000)) as nyhed_content
	from trader.nyheds_lager;
  
begin
	util_run_compare.middle_compare('morten@egan.dk', 'name:Insert with compression');
end;
/

create table do_compress (
	nyhed_id	number,
	nyhed_titel	varchar2(4000),
	nyhed_content	blob
);

insert into do_compress
	select nyhed_id, nyhed_titel, do_compress_content(nyhed_content)
	from no_compress;

commit;

begin
	util_run_compare.end_compare('morten@egan.dk', 'name:Insert with compression');
end;
/

set serverout on

begin
	sys.show_space('NO_COMPRESS');
end;
/

begin
	sys.show_space('DO_COMPRESS');
end;
/

create or replace type do_compress_r
as object (
	nyhed_id	number,
	nyhed_titel	varchar2(4000),
	nyhed_content	varchar2(4000)
);
/

create or replace type do_compress_tab
as table of do_compress_r
/

create or replace function get_compressed_content
return do_compress_tab
pipelined

as

	cursor get_content is
		select
			nyhed_id
			, nyhed_titel
			, nyhed_content
		from
			do_compress;

	l_blob			blob := empty_blob;
	ret_text		varchar2(4000);

begin

	for x in get_content loop
		dbms_lob.createtemporary(l_blob, true);
		l_blob := utl_compress.lz_uncompress(x.nyhed_content);
		ret_text := utl_raw.cast_to_varchar2(l_blob);
		dbms_lob.freetemporary(l_blob);

		pipe row(do_compress_r(x.nyhed_id, x.nyhed_titel, ret_text));
	end loop;

	return;

end get_compressed_content;
/

begin
	util_run_compare.start_compare('morten@egan.dk', 'title:REDGATE Compression select;name:Select from uncompressed');
end;
/

select nyhed_id, nyhed_content from no_compress;

begin
	util_run_compare.middle_compare('morten@egan.dk', 'name:Select from compressed');
end;
/

select nyhed_id, nyhed_content from table(get_compressed_content);

begin
	util_run_compare.end_compare('morten@egan.dk', 'name:Select from compressed');
end;
/